@@ -286,7 +286,7 @@ namespace clx {
 			/* ----------------------------------------------------------------- */
 			//  send
 			/* ----------------------------------------------------------------- */
-			virtual int send(const char_type* src, int n) {
+			int send(const char_type* src, int n) {
 				if (!p_ || !p_->ssl()) return -1;
 				return SSL_write(p_->ssl(), src, n);
 			}
@@ -294,18 +294,26 @@ namespace clx {
 			/* ----------------------------------------------------------------- */
 			//  send
 			/* ----------------------------------------------------------------- */
-			virtual int send(const string_type& src) {
+			int send(const string_type& src) {
 				return this->send(src.c_str(), static_cast<int>(src.size()));
 			}
 			
 			/* ----------------------------------------------------------------- */
 			//  recv
 			/* ----------------------------------------------------------------- */
-			virtual int recv(char_type* dest, int n) {
+			int recv(char_type* dest, int n) {
 				if (!p_ || !p_->ssl()) return -1;
 				return SSL_read(p_->ssl(), dest, n);
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  recv
+			/* ----------------------------------------------------------------- */
+			template <std::size_t N>
+			int recv(char_type (&dest)[N]) {
+				return this->recv(dest, N);
+			}
+			
 			/* ----------------------------------------------------------------- */
 			//  close
 			/* ----------------------------------------------------------------- */
