@@ -51,45 +51,46 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		template <int Family>
 		class basic_socket : public basic_rawsocket<SOCK_STREAM, Family, 0> {
+		private:
+			typedef basic_rawsocket<SOCK_STREAM, Family, 0> super;
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
-			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
+			typedef basic_sockaddress<Family, IPPROTO_TCP> address_type;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
 			
-			basic_socket() : rawsocket(), addr_(), connect_(false) {}
+			basic_socket() : super(), addr_(), connect_(false) {}
 			
 			basic_socket(const basic_socket& cp) :
-				rawsocket(cp), addr_(cp.addr_), connect_(cp.connect_) {}
+				super(cp), addr_(cp.addr_), connect_(cp.connect_) {}
 			
 			// for client socket
-			explicit basic_socket(const char_type* host, int port) :
-				rawsocket(), addr_(host, port), connect_(false) {
-				this->xconnect();
+			basic_socket(const char_type* host, int port) :
+				super(), addr_(host, port), connect_(false) {
+				this->connect();
 			}
 			
-			explicit basic_socket(const string_type& host, int port) :
-				rawsocket(), addr_(host, port), connect_(false) {
-				this->xconnect();
+			basic_socket(const string_type& host, int port) :
+				super(), addr_(host, port), connect_(false) {
+				this->connect();
 			}
 			
 			// for server socket
-			explicit basic_socket(socket_int s, const sockaddress& addr) :
-				rawsocket(s), addr_(addr), connect_(true) {}
+			basic_socket(socket_int s, const address_type& addr) :
+				super(s), addr_(addr), connect_(true) {}
 			
 			virtual ~basic_socket() throw() {}
 			
 			basic_socket& operator=(const basic_socket& cp) {
 				addr_ = cp.addr_;
 				connect_ = cp.connect_;
-				this->assign(static_cast<rawsocket>(cp));
+				this->assign(static_cast<super>(cp));
 				return *this;
 			}
 			
 			// operator
 			basic_socket& connect(const char_type* host, int port) {
 				addr_.assign(host, port);
-				return this->xconnect();
+				return this->connect();
 			}
 			
 			basic_socket& connect(const string_type& host, int port) {
@@ -97,29 +98,34 @@ namespace clx {
 			}
 			
 			virtual void close() {
-				rawsocket::close();
+				super::close();
 				connect_ = false;
 			}
 			
-			virtual int send(const char_type* src, int n) {
+			int send(const char_type* src, int n) {
 				return ::send(this->socket(), src, n, 0);
 			}
 			
-			virtual int send(const string_type& src) {
+			int send(const string_type& src) {
 				return this->send(src.c_str(), static_cast<int>(src.size()));
 			}
 			
-			virtual int recv(char_type* dest, int n) {
+			int recv(char_type* dest, int n) {
 				return ::recv(this->socket(), dest, n, 0);
 			}
 			
-			const sockaddress& address() const { return addr_; }
+			template <std::size_t N>
+			int recv(char_type (&dest)[N]) {
+				return this->recv(dest, N);
+			}
+			
+			const address_type& address() const { return addr_; }
 			bool is_connect() const { return connect_; }
 		private:
-			sockaddress addr_;
+			address_type addr_;
 			bool connect_;
 			
-			basic_socket& xconnect() {
+			basic_socket& connect() {
 				if (connect_) return *this;
 				if (!this->is_open()) this->open();
 				if (!this->is_open()) throw clx::socket_error("socket");
@@ -144,20 +150,21 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		template <int Family>
 		class basic_acceptor : public basic_rawsocket<SOCK_STREAM, Family, 0> {
+		private:
+			typedef basic_rawsocket<SOCK_STREAM, Family, 0> super;
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
-			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
+			typedef basic_sockaddress<Family, IPPROTO_TCP> address_type;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
 			
 			enum { nmax = 100 };
 			
 			// constructor and destructor
-			basic_acceptor() : rawsocket(), nlisten_(nmax) {}
+			basic_acceptor() : super(), nlisten_(nmax) {}
 			
 			explicit basic_acceptor(int port, int n = nmax) :
-				rawsocket(), addr_(port), nlisten_(nmax) {
-				this->xopen();
+				super(), addr_(port), nlisten_(nmax) {
+				this->start();
 			}
 			
 			virtual ~basic_acceptor() throw() {}
@@ -166,13 +173,13 @@ namespace clx {
 			basic_acceptor& open(int port, int n = nmax) {
 				addr_.assign(port);
 				nlisten_ = n;
-				return this->xopen();
+				return this->start();
 			}
 			
 			basic_socket<Family> accept() {
 				socket_int s = -1;
-				sockaddress addr;
-				typename sockaddress::inet_type tmp;
+				address_type addr;
+				typename address_type::inet_type tmp;
 				socklen_type len = sizeof(tmp);
 				
 				for (int foo = 0; foo < 10; ++foo) {
@@ -180,16 +187,16 @@ namespace clx {
 					if (s == -1) clx::sleep(1.0);
 					else break;
 				}
-				addr = tmp;
+				addr.assign(tmp);
 				return basic_socket<Family>(s, addr);
 			}
 		
 		private:
-			sockaddress addr_;
+			address_type addr_;
 			int nlisten_;
 			
-			basic_acceptor& xopen() {
-				if (rawsocket::open() < 0) throw socket_error("socket");
+			basic_acceptor& start() {
+				if (super::open() < 0) throw socket_error("socket");
 				if (::bind(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
 					throw socket_error("bind");
 				}
